from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4.QtSql import *

class EditPerson(QWidget):
	def __init__(self, parent=None, db=None):
		"""Set up EditPerson with parent widget and database from which to pull data"""
		QWidget.__init__(self,parent)
		self.db = db

		self.setupModel()

		self.tableview = QTableView(self)
		self.tableview.setSelectionBehavior(QAbstractItemView.SelectRows)
		self.tableview.setSelectionMode(QAbstractItemView.SingleSelection)
		self.tableview.setEditTriggers(QAbstractItemView.NoEditTriggers)
		self.tableview.setModel(self.model)

		for i in xrange(self.model.columnCount()):
			self.tableview.hideColumn(i)
		self.tableview.showColumn(self.model.fieldIndex("forename"))
		self.tableview.showColumn(self.model.fieldIndex("surname"))
		self.tableview.resizeColumnsToContents()

		self.setWindowTitle("Edit records")
		self.toplayout = QHBoxLayout() # Master layout
		
		self.leftpanel = QVBoxLayout() # Contains user search and select
		self.search_label = QLabel("F&ilter by name:")
		self.search_bar = QLineEdit()
		self.search_label.setBuddy(self.search_bar)
		self.searchpair = QHBoxLayout()
		self.searchpair.addWidget(self.search_label)
		self.searchpair.addWidget(self.search_bar)
		self.leftpanel.addLayout(self.searchpair)
		self.leftpanel.addWidget(self.tableview)
		
		self.rightpanel = QVBoxLayout() # Contains fields and buttons
		self.setupForm()


		self.rightpanel.addLayout(self.formlayout)
		self.rightpanel.addLayout(self.buttonlayout)

		self.toplayout.addLayout(self.leftpanel)
		self.toplayout.addSpacing(10)
		self.toplayout.addLayout(self.rightpanel)

		self.setLayout(self.toplayout)
		self.setupActions()
		self.resize(800,600)
		self.show()
		
	def setupModel(self):
		"""Prepare data model for the table on the left and add header fields"""
		self.model = QSqlTableModel(self, self.db)
		self.model.setEditStrategy(QSqlTableModel.OnManualSubmit)
		self.model.setTable("people")
		self.model.select()
		self.model.setHeaderData(self.model.fieldIndex("forename"), Qt.Horizontal, QVariant("First Name"))
		self.model.setHeaderData(self.model.fieldIndex("surname"), Qt.Horizontal, QVariant("Last Name"))
	def setupForm(self):
		"""Initialize data breakout widgets on right"""
		self.current_record = QSqlRecord()
		self.current_row = -1
		# Text labels
		self.netid_lab = QLabel("NetID:")
		self.year_lab = QLabel("Year:")
		self.month_lab = QLabel("Month:")
		self.day_lab = QLabel("Day:")
		self.firstname_lab = QLabel("Preferred name:")
		self.surname_lab = QLabel("Last name:")
		self.email_lab = QLabel("Email:")
		self.phone_lab = QLabel("Phone:")
		self.major_lab = QLabel("Major:")
		self.hometown_lab = QLabel("Hometown:")
		self.room_lab = QLabel("Room number:")
		self.bday_lab = QLabel("Birthday:")
		self.dorm_lab = QLabel("Dorm:")
		self.photo = QLabel("No record selected")
		self.photo.setScaledContents(False)
		self.currentimage = QImage()
		self.displayimage = QImage()
		self.displaypixmap = QPixmap()
		self.photo.setAlignment(Qt.AlignCenter)
		self.pb_changephoto = QPushButton("Select &photo")
		self.pb_changephoto.setEnabled(False)
		# Personal info
		self.surname = QLineEdit()
		self.firstname = QLineEdit()
		self.phone = QLineEdit()
		self.email = QLineEdit()
		self.major = QLineEdit()
		self.netid = QLineEdit()
		# Birthday
		self.year = QComboBox()
		self.month = QComboBox()
		self.day = QComboBox()
		# Residence locations
		self.dorm = QComboBox()
		self.room = QLineEdit()
		# Fill the comboboxes with values, set valid field types
		years = QStringList(["1983","1984","1985","1986","1987","1988","1989","1990","1991","1992","1993","1994"])
		months = QStringList(["January","February","March","April","May","June","July","August","September","October","November","December"])
		days = QStringList(["1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31"])
		dorms = QStringList(["Clements","Lechner","McFadden"])
		self.year.addItems(years)
		self.month.addItems(months)
		self.day.addItems(days)
		self.dorm.addItems(dorms)
		self.phone.setInputMask("(999)-999-9999")
		self.major.setMaxLength(4)
		self.room.setMaxLength(3)
		# Create and fill the UI
		self.formlayout = QGridLayout()
		self.bday_layout = QHBoxLayout()
		self.bday_layout.addWidget(self.year)
		self.bday_layout.addWidget(self.month)
		self.bday_layout.addWidget(self.day)
		self.formlayout.addWidget(self.netid_lab, 0, 0)
		self.formlayout.addWidget(self.netid, 0, 1)
		self.formlayout.addWidget(self.firstname_lab, 1, 0)
		self.formlayout.addWidget(self.firstname, 1, 1)
		self.formlayout.addWidget(self.surname_lab, 2, 0)
		self.formlayout.addWidget(self.surname, 2, 1)
		self.formlayout.addWidget(self.email_lab, 3, 0)
		self.formlayout.addWidget(self.email, 3, 1)
		self.formlayout.addWidget(self.phone_lab, 4, 0)
		self.formlayout.addWidget(self.phone, 4, 1)
		self.formlayout.addWidget(self.bday_lab,5,0)
		self.formlayout.addLayout(self.bday_layout,5,1)
		self.formlayout.addWidget(self.major_lab, 6, 0)
		self.formlayout.addWidget(self.major, 6, 1)
		self.formlayout.addWidget(self.dorm_lab, 7, 0)
		self.formlayout.addWidget(self.dorm, 7, 1)
		self.formlayout.addWidget(self.room_lab, 8, 0)
		self.formlayout.addWidget(self.room, 8, 1)
		self.formlayout.addWidget(self.photo,9,0,1,2)
		self.formlayout.addWidget(self.pb_changephoto,10,0,1,2)
		self.buttonlayout = QHBoxLayout()
		self.pb_cancel = QPushButton("&Cancel")
		self.pb_save = QPushButton("&Save")
		self.pb_saveclose = QPushButton("Save && Clos&e")
		self.pb_reset = QPushButton("&Reset")
		self.pb_save.setEnabled(False)
		self.pb_saveclose.setEnabled(False)
		self.pb_reset.setEnabled(False)
		self.buttonlayout.addWidget(self.pb_reset)
		self.buttonlayout.addWidget(self.pb_cancel)
		self.buttonlayout.addWidget(self.pb_save)
		self.buttonlayout.addWidget(self.pb_saveclose)
	def setupActions(self):
		"""Set up signal/slot connections"""
		# Update filtered table
		QObject.connect(self.search_bar, SIGNAL("textChanged(QString)"), self.updateTable )
		# Selecting a record loads it for editing
		QObject.connect(self.tableview.selectionModel(), SIGNAL("selectionChanged(QItemSelection, QItemSelection)"), self.selectionChanged )
		# Map buttons to actions
		QObject.connect(self.pb_reset, SIGNAL("clicked()"), self.resetPressed )
		QObject.connect(self.pb_cancel, SIGNAL("clicked()"), self.closePressed )
		QObject.connect(self.pb_saveclose, SIGNAL("clicked()"), self.saveClosePressed )
		QObject.connect(self.pb_save, SIGNAL("clicked()"), self.savePressed )
		QObject.connect(self.pb_changephoto, SIGNAL("clicked()"), self.changePhoto )
		# Changes to the record enable buttons
		QObject.connect(self.netid, SIGNAL("textEdited(QString)"), self.formChanged )
		QObject.connect(self.firstname, SIGNAL("textEdited(QString)"), self.formChanged )
		QObject.connect(self.surname, SIGNAL("textEdited(QString)"), self.formChanged )
		QObject.connect(self.email, SIGNAL("textEdited(QString)"), self.formChanged )
		QObject.connect(self.phone, SIGNAL("textEdited(QString)"), self.formChanged )
		QObject.connect(self.major, SIGNAL("textEdited(QString)"), self.formChanged )
		QObject.connect(self.room, SIGNAL("textEdited(QString)"), self.formChanged )
		QObject.connect(self.year, SIGNAL("currentIndexChanged(int)"), self.formChanged )
		QObject.connect(self.month, SIGNAL("currentIndexChanged(int)"), self.formChanged )
		QObject.connect(self.day, SIGNAL("currentIndexChanged(int)"), self.formChanged )
		QObject.connect(self.dorm, SIGNAL("currentIndexChanged(int)"), self.formChanged )

	def formChanged(self):
		"""Enable Save/Reset buttons when a field has been changed in the editor"""
		# Do this only if a record has been selected
		if not self.current_record.isEmpty():
			self.pb_reset.setEnabled(True)
			self.pb_save.setEnabled(True)
			self.pb_saveclose.setEnabled(True)
	def selectionChanged(self, selected, deselected):
		"""Handle changes in table row selection"""
		items = selected.indexes()
		if len(items) > 0:
			self.current_row = items[0].row() # this is the row of the current selected item
			self.current_record = self.model.record(items[0].row())
			self.fillForm()
	def fillForm(self):
		"""Fill form with fields from the selected row of the database"""
		# Fill fields with data from selected record
		self.netid.setText(self.current_record.field("netid").value().toString())
		self.firstname.setText(self.current_record.field("forename").value().toString())
		self.surname.setText(self.current_record.field("surname").value().toString())
		self.email.setText(self.current_record.field("email").value().toString())
		self.phone.setText(self.current_record.field("phone").value().toString())
		self.major.setText(self.current_record.field("major").value().toString())
		self.room.setText(self.current_record.field("room").value().toString())
		bday = str(self.current_record.field("birthday").value().toString()).split("-")
		self.year.setCurrentIndex(self.year.findText(bday[0]))
		self.month.setCurrentIndex(int(bday[1]) - 1)
		self.day.setCurrentIndex(int(bday[2]) - 1)
		self.dorm.setCurrentIndex(self.dorm.findText(self.current_record.field("dorm").value().toString()))
		self.pb_reset.setEnabled(False)
		self.pb_save.setEnabled(False)
		self.pb_saveclose.setEnabled(False)
		ba = self.current_record.field("photo").value().toByteArray()
		if ba.size() > 0:
			self.currentimage = QImage()
			self.currentimage.loadFromData(ba)
			self.displayimage = self.currentimage.scaled(self.photo.size(), Qt.KeepAspectRatio, Qt.SmoothTransformation)
			self.displaypixmap = QPixmap.fromImage(self.displayimage)
			self.photo.setPixmap(self.displaypixmap)
		else:
			self.photo.setText("No photo available")
		self.pb_changephoto.setEnabled(True)

	def reselect(self):
		"""Refreshes the table view from the data source.  Called when databases merged - need to reload rows from database."""
		self.updateTable(self.search_bar.text())
	def updateTable(self, text):
		"""Filter table view with text in the search bar.  Table view shows only entries that contain each of the space-separated strings in the bar in either the entries forename or surname."""
		if text.isEmpty():
			self.model.setFilter("")
			self.model.select()
			return
		t = str(text)
		strs = t.split()
		patterns = [] # can't use list comprehensions for this one because setValue returns void
		for i in strs:
			f = QSqlField("input",QVariant.String)
			f.setValue(QVariant(QString("%" + i + "%")))
			patterns.append(f)
		escapedpatterns = [ self.model.database().driver().formatValue(wc, False) for wc in patterns]
		filters = [ str(QString("(surname LIKE %1 OR forename LIKE %2)").arg(wc).arg(wc)) for wc in escapedpatterns]
		filterString = " AND ".join(filters)
		self.model.setFilter(QString(filterString))
		self.model.select()
		if self.model.rowCount() == 1:
			self.tableview.selectRow(0)
	def resetPressed(self):
		"""Revert fields in editor to their source from the database.  Disable Save/Reset buttons."""
		self.current_record = self.model.record(self.current_row)
		self.fillForm()
		self.pb_reset.setEnabled(False)
		self.pb_save.setEnabled(False)
		self.pb_saveclose.setEnabled(False)
		print "Resetting form"
	def closePressed(self):
		"""Revert all unsaved changes, clear all entries, and emit done() for parent to swap widgets shown."""
		print "Closing"
		self.model.revertAll()
		# Wipe contents of all lineedits, destroy current_record and current_index
		self.wipeFields()
		self.model.select()
		self.emit(SIGNAL("done()"))
	def savePressed(self):
		"""Commit changes to currently-selected record to the database."""
		print "Saving"
		self.current_record.setValue("netid",QVariant(self.netid.text()))
		self.current_record.setValue("forename",QVariant(self.firstname.text()))
		self.current_record.setValue("surname",QVariant(self.surname.text()))
		self.current_record.setValue("email",QVariant(self.email.text()))
		self.current_record.setValue("phone",QVariant(self.phone.text()))
		self.current_record.setValue("major",QVariant(self.major.text()))
		self.current_record.setValue("room",QVariant(self.room.text()))
		self.current_record.setValue("dorm",QVariant(self.dorm.currentText()))
		y = self.year.currentText().toInt()[0]
		m = self.month.currentIndex() + 1
		d = self.day.currentText().toInt()[0]
		bday = QDate(y,m,d)
		self.current_record.setValue("birthday", QVariant(bday.toString(Qt.ISODate)) )
		self.current_record.setValue("mtime", QVariant(QDateTime.currentDateTime().toString(Qt.ISODate)) ) # update modification time

		self.model.setRecord(self.current_row, self.current_record)
		self.model.submitAll()
		self.updateTable(self.search_bar.text()) # Reselect from the DB
		self.pb_reset.setEnabled(False)
		self.pb_save.setEnabled(False)
		self.pb_saveclose.setEnabled(False)
	def saveClosePressed(self):
		"""Act as if Save were pressed, then Close."""
		self.savePressed()
		self.closePressed()
	def changePhoto(self):
		"""Allow user to pick a JPEG or PNG file to set as the current record's photo."""
		fileName = QFileDialog.getOpenFileName(self, "Select Photo", ".", "Images (*.jpg *.JPG *.png)" )
		if not fileName.isEmpty():
			print "opening", fileName
			self.currentimage = QImage()
			self.currentimage.load(fileName)
			self.displayimage = self.currentimage.scaled(self.photo.size(), Qt.KeepAspectRatio, Qt.SmoothTransformation)
			self.displaypixmap = QPixmap.fromImage(self.displayimage)
			self.photo.setPixmap(self.displaypixmap)
			ba = QByteArray()
			buffer = QBuffer(ba)
			buffer.open(QIODevice.WriteOnly)
			self.currentimage.save(buffer, "JPG")
			buffer.close()
			self.current_record.setValue("photo",QVariant(ba))
			self.pb_reset.setEnabled(True)
			self.pb_save.setEnabled(True)
			self.pb_saveclose.setEnabled(True)
	def wipeFields(self):
		"""Clear all editable fields, including the search bar."""
		self.search_bar.clear()
		self.current_record = QSqlRecord()
		self.current_index = -1
		self.surname.clear()
		self.firstname.clear()
		self.phone.clear()
		self.email.clear()
		self.major.clear()
		self.netid.clear()
		self.year.setCurrentIndex(0)
		self.month.setCurrentIndex(0)
		self.day.setCurrentIndex(0)
		self.dorm.setCurrentIndex(0)
		self.room.clear()
		self.photo.setText("No record selected")

	def updateDB(self, newdbname):
		"""Force database reload with new database name."""
		print "updateDB called",newdbname
		self.db.close()
		self.db.setDatabaseName(newdbname)
		self.db.open()
		self.setupModel()
		self.tableview.setModel(self.model)
		QObject.connect(self.tableview.selectionModel(), SIGNAL("selectionChanged(QItemSelection, QItemSelection)"), self.selectionChanged )
		self.wipeFields()
		self.updateTable(QString())



if __name__ == "__main__":
	a = QApplication([""])
	w = EditPerson()
	a.exec_()
