from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4.QtSql import *

from chooseaction import ChooseAction
from editperson import EditPerson
from mergephotos import MergePhotos
from odtwriter import ODTWriter
from analytics import Analytics

class MainApp (QMainWindow):
	schema = "CREATE TABLE people ( id INTEGER PRIMARY KEY AUTOINCREMENT, forename TEXT NOT NULL, surname TEXT NOT NULL, netid TEXT NOT NULL, email TEXT, birthday TEXT, phone TEXT, major TEXT, dorm TEXT, room INTEGER, photo BLOB, createtime TEXT NOT NULL, mtime TEXT NOT NULL);"
	def __init__(self, Parent=None):
		"""Set up the main view, load the default database and schema, and set the widgets visible."""
		QMainWindow.__init__(self, Parent)
		self.setWindowTitle("Whitepages V3")

		self.db = QSqlDatabase.addDatabase("QSQLITE")
		self.db.setDatabaseName("families.db")
		self.db.open()
		q = QSqlQuery(self.db)
		q.exec_(MainApp.schema)

		self.center = QStackedWidget()
		self.chooseaction = ChooseAction(self,self.db)
		self.editperson = EditPerson(self,self.db)
		self.mergephotos = MergePhotos(self,self.db)
		self.analytics = Analytics(self,self.db)
		self.center.addWidget(self.chooseaction)
		self.center.addWidget(self.editperson)
		self.center.addWidget(self.mergephotos)
		self.center.addWidget(self.analytics)
		self.setCentralWidget(self.center)

		self.createActions()
		self.createMenus()

		self.statusBar().showMessage("Ready")
		self.setUnifiedTitleAndToolBarOnMac(True);
		self.resize(800,600)
		self.show()

	def createActions(self):
		"""Create actions and connect signals and slots."""
		self.fileNewAction = QAction("&New Database",self)
		QObject.connect( self.fileNewAction, SIGNAL("triggered()"), self.newFile)
		self.fileOpenAction = QAction("&Open Database",self)
		QObject.connect( self.fileOpenAction, SIGNAL("triggered()"), self.openFile)
		self.fileImportAction = QAction("&Import (Merge)", self)
		QObject.connect( self.fileImportAction, SIGNAL("triggered()"), self.mergeWizard)
		self.fileQuitAction = QAction("&Quit",self)
		QObject.connect( self.fileQuitAction, SIGNAL("triggered()"), self.quit)
		QObject.connect( self.chooseaction, SIGNAL("editPerson()"), self.editPersonSlot)
		QObject.connect( self.chooseaction, SIGNAL("mergePhotos()"), self.mergePhotosSlot)
		QObject.connect( self.chooseaction, SIGNAL("exportDocument(QString)"), self.exportDocumentSlot)
		QObject.connect( self.chooseaction, SIGNAL("runAnalytics()"), self.runAnalyticsSlot)
		QObject.connect( self.editperson, SIGNAL("done()"), self.returnToMainMenu)
		QObject.connect( self.mergephotos, SIGNAL("done()"), self.returnToMainMenu)
		QObject.connect( self.analytics, SIGNAL("done()"), self.returnToMainMenu)
		QObject.connect( self, SIGNAL("databaseChanged(QString)"), self.editperson.updateDB)
		QObject.connect( self, SIGNAL("databaseChanged(QString)"), self.analytics.updateDB)
		QObject.connect( self, SIGNAL("mergeDone()"), self.editperson.reselect )
		
	
	def createMenus(self):
		"""Create the drop-down File menu."""
		self.fileMenu = self.menuBar().addMenu("&File")
		self.fileMenu.addAction(self.fileNewAction)
		self.fileMenu.addAction(self.fileOpenAction)
		self.fileMenu.addAction(self.fileImportAction)
		self.fileMenu.addSeparator()
		self.fileMenu.addAction(self.fileQuitAction)


	def newFile(self):
		"""Allow user to specify a new SQLite database to work with."""
		fileName = QFileDialog.getSaveFileName(self, "Save new file as:", ".", "Databases (*.db)")
		if not fileName.isEmpty(): # If they cancelled, do nothing
			if not fileName.endsWith(".db"): # if they leave off the extension, add it
				fileName = fileName.append(".db")
			self.db.close()
			self.db.setDatabaseName(fileName)
			self.db.open()
			q = QSqlQuery(self.db)
			q.exec_(MainApp.schema)
			self.statusBar().showMessage("Closed the existing database, created " + fileName)
			self.emit(SIGNAL("databaseChanged(QString)"),self.db.databaseName())
		else:
			self.statusBar().showMessage("Cancelled creating new database")
	def openFile(self):
		"""Load user-specified SQLite database for editing."""
		fileName = QFileDialog.getOpenFileName(self, "Open dataset", ".", "Databases (*.db)")
		if not fileName.isEmpty():
			self.db.close()
			self.db.setDatabaseName(fileName)
			self.db.open()
			self.statusBar().showMessage("Successfully loaded database " + fileName)
			self.returnToMainMenu()
			self.emit(SIGNAL("databaseChanged(QString)"),self.db.databaseName())
		else:
			self.statusBar().showMessage("Canceled loading database")
	def mergeWizard(self):
		"""Import user-specified SQLite database into this database, with intelligent merge heuristics.  NetID conflicts are resolved by ignoring the to-be-imported record and keeping the one in the currently-open database."""
		fileName = QFileDialog.getOpenFileName(self, "Import which dataset?", ".", "Databases (*.db)" )
		if not fileName.isEmpty():
			tempdb = QSqlDatabase.addDatabase("QSQLITE","tempdb")
			tempdb.setDatabaseName(fileName)
			tempdb.open()
			q = QSqlQuery(self.db)
			import_q = QSqlQuery(tempdb)
			import_q.exec_("SELECT * FROM people;")
			rec = import_q.record()
			col_id = rec.indexOf("id")
			col_forename = rec.indexOf("forename")
			col_surname = rec.indexOf("surname")
			col_netid = rec.indexOf("netid")
			col_email = rec.indexOf("email")
			col_birthday = rec.indexOf("birthday")
			col_phone = rec.indexOf("phone")
			col_major = rec.indexOf("major")
			col_dorm = rec.indexOf("dorm")
			col_room = rec.indexOf("room")
			col_photo = rec.indexOf("photo")
			col_createtime = rec.indexOf("createtime")
			col_mtime = rec.indexOf("mtime")
			#print "Number of columns:",rec.count()
			records_merged = 0
			while import_q.next():
				qtestnewnetid = QSqlQuery(self.db)
				qtestnewnetid.prepare("SELECT * FROM people WHERE netid = :netid")
				qtestnewnetid.bindValue(":netid", import_q.value(col_netid) )
				qtestnewnetid.exec_()
				if qtestnewnetid.next(): # if the person's already in the DB, keep the old data
					continue
				q.prepare("INSERT INTO people (netid, forename, surname, email, birthday, phone, major, dorm, room, createtime, mtime, photo )"
								"VALUES (:netid, :forename, :surname, :email, :birthday, :phone, :major, :dorm, :room, :createtime, :mtime, :photo )" )
				q.bindValue(":netid", import_q.value(col_netid) )
				q.bindValue(":forename", import_q.value(col_forename) )
				q.bindValue(":surname", import_q.value(col_surname) )
				q.bindValue(":email", import_q.value(col_email) )
				q.bindValue(":birthday", import_q.value(col_birthday) )
				q.bindValue(":phone", import_q.value(col_phone) )
				q.bindValue(":major", import_q.value(col_major) )
				q.bindValue(":dorm", import_q.value(col_dorm) )
				q.bindValue(":room", import_q.value(col_room) )
				q.bindValue(":createtime", import_q.value(col_createtime) )
				q.bindValue(":mtime", import_q.value(col_mtime) )
				q.bindValue(":photo", import_q.value(col_photo) )
				q.exec_()
				print "Inserted", import_q.value(col_forename).toString(), import_q.value(col_surname).toString(), "into DB"
				records_merged = records_merged + 1
			self.statusBar().showMessage(QString("Successfully merged %1 records from %2").arg(QString(str(records_merged))).arg(fileName))
			tempdb.close()
			self.emit(SIGNAL("mergeDone()"))
		else:
			self.statusBar().showMessage("Canceled merging databases")
	def quit(self):
		"""Exit program."""
		# You could do cleanup, like closing/flushing the database, an "Are you sure you want to quit?"
		# modal dialog, or saving the window layout/state.
		qApp.quit()
	def returnToMainMenu(self):
		"""Switch to ChooseAction widget."""
		self.center.setCurrentWidget(self.chooseaction)
	def editPersonSlot(self):
		"""Switch to EditPerson widget."""
		self.editperson.model.select() # update the table
		self.center.setCurrentWidget(self.editperson)
	def mergePhotosSlot(self):
		"""Switch to MergePhotos widget."""
		print "beginning photo merge"
		self.center.setCurrentWidget(self.mergephotos)
		#self.mergephotos.beginMerge()
	def exportDocumentSlot(self, filename):
		"""Export current database to ODT file <filename>."""
		writer = ODTWriter(self.db)
		success = writer.write(filename)
		if success:
			self.statusBar().showMessage(QString("Successfully exported to %1").arg(filename))
		else:
			self.statusBar().showMessage(QString("Document export cancelled or failed"))
	def runAnalyticsSlot(self):
		"""Switch to Analytics widget."""
		print "Running analytics"
		self.analytics.generateReport()
		self.center.setCurrentWidget(self.analytics)


if __name__ == "__main__" :
	app = QApplication([""])
	widget = MainApp()
	app.exec_()

