from PyQt4.QtCore import *
from PyQt4.QtGui import *

from ldapsearcher import LDAPSearcher

class PageNewUserData(QWizardPage):
	def __init__(self, parent=None):
		"""Set up lots of input fields."""
		QWizardPage.__init__(self,parent)
		self.setCommitPage(True)
		self.setButtonText(QWizard.CommitButton, "Next")
		# Create all the subwidgets
		self.year_lab = QLabel("Year:")
		self.month_lab = QLabel("Month:")
		self.day_lab = QLabel("Day:")
		self.forename_lab = QLabel("Preferred name:")
		self.surname_lab = QLabel("Last name:")
		self.email_lab = QLabel("Email:")
		self.phone_lab = QLabel("Phone:")
		self.major_lab = QLabel("Major:")
		self.hometown_lab = QLabel("Hometown:")
		self.room_lab = QLabel("Room number:")
		self.bday_lab = QLabel("Birthday:")
		self.dorm_lab = QLabel("Hall:")
		# Personal info
		self.surname = QLineEdit()
		self.forename = QLineEdit()
		self.phone = QLineEdit()
		self.email = QLineEdit()
		self.major = QLineEdit()
		# Birthday
		self.year = QComboBox()
		self.month = QComboBox()
		self.day = QComboBox()
		# Residence locations
		self.dorm = QComboBox()
		self.room = QLineEdit()
		# Fill the comboboxes with values, set valid field types
		years = QStringList(["1983","1984","1985","1986","1987","1988","1989","1990","1991","1992","1993","1994"])
		months = QStringList(["January","February","March","April","May","June","July","August","September","October","November","December"])
		days = QStringList(["1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31"])
		dorms = QStringList(["","Clements","Lechner","McFadden"])
		self.year.addItems(years)
		self.month.addItems(months)
		self.day.addItems(days)
		self.year.setCurrentIndex(6) # Start in 1988 or so
		self.dorm.addItems(dorms)
		self.phone.setInputMask("(999)-999-9999")
		self.major.setMaxLength(4)
		self.room.setMaxLength(3)

		# Register fields with QWizard
		self.registerField("surname*", self.surname)
		self.registerField("forename*", self.forename)
		self.registerField("phone", self.phone)
		self.registerField("email*", self.email)
		self.registerField("major", self.major)
		self.registerField("dorm*", self.dorm)
		self.registerField("room*", self.room)
		self.registerField("year", self.year)
		self.registerField("month", self.month)
		self.registerField("day", self.day)
		

		# Set up layout
		self.layout = QGridLayout()
		self.bday_layout = QHBoxLayout()
		self.bday_layout.addWidget(self.year)
		self.bday_layout.addWidget(self.month)
		self.bday_layout.addWidget(self.day)
		self.layout.addWidget(self.forename_lab, 1, 0)
		self.layout.addWidget(self.forename, 1, 1)
		self.layout.addWidget(self.surname_lab, 2, 0)
		self.layout.addWidget(self.surname, 2, 1)
		self.layout.addWidget(self.email_lab, 3, 0)
		self.layout.addWidget(self.email, 3, 1)
		self.layout.addWidget(self.phone_lab, 4, 0)
		self.layout.addWidget(self.phone, 4, 1)
		self.layout.addWidget(self.bday_lab,5,0)
		self.layout.addLayout(self.bday_layout,5,1)
		self.layout.addWidget(self.major_lab, 6, 0)
		self.layout.addWidget(self.major, 6, 1)
		self.layout.addWidget(self.dorm_lab, 7, 0)
		self.layout.addWidget(self.dorm, 7, 1)
		self.layout.addWidget(self.room_lab, 8, 0)
		self.layout.addWidget(self.room, 8, 1)
		self.setLayout(self.layout)
		self.setTitle("Move-in Wizard")
		self.setSubTitle("Enter your contact information:")


	def initializePage(self):
		"""Load known information from LDAP, like email, major, name, and NetID."""
		a = LDAPSearcher()
		ldapinfo = a.lookup(str(self.field("netid").toString()))
		for key,value in ldapinfo.iteritems():
			if(key == "sn"):
				self.surname.setText(value)
			elif(key == "givenName"):
				self.forename.setText(value)
			elif(key == "mail"):
				self.email.setText(value)
			elif(key == "major"):
				self.major.setText(value)
			elif(key == "tamuEduPersonNetID"):
				self.setField("netid",QVariant(QString(value)))


if __name__ == "__main__":
	a = QApplication([""])
	wiz = QWizard()
	p = PageNewUserData()
	wiz.addPage(p)
	wiz.resize(400,300)
	wiz.show()
	a.exec_()

